@@ -1,12 +1,12 @@
 # -*- coding: utf-8 -*-
 # package: pypos3d.pftk
 import os
-import sys
+import sys, traceback
 import logging
 from collections import namedtuple, UserDict
 from bisect import bisect_left
 
-from langutil import C_OK, C_FAIL, C_ERROR, C_UNKNOWN, C_FILE_NOT_FOUND
+from langutil import C_OK, C_FAIL, C_ERROR, C_UNKNOWN, C_FILE_NOT_FOUND, UsageError
 
 from pypos3d.wftk.PoserFileParser import PoserFileParser
 
@@ -19,8 +19,6 @@ from pypos3d.pftk.ChannelImportAnalysis import FigurePart, ChannelImportAnalysis
 from pypos3d.wftk.WaveGeom import WaveGeom, readGeom
 
 
-
-
 # Internal tuple used to store addChild and weld links between figure's parts
 PMOLink = namedtuple('PMOLink', ['son', 'parent', 'sonPMO', 'parentPMO'])
 
@@ -1544,6 +1542,10 @@ class Figure():
   
  
   def __copyMat(self, fp, lstMat):
+    if not fp.name:
+      fp.status(C_FAIL, 'Material Source Undefined is illegal in {:s}', fp.geomGroup.getName())
+      return
+    
     if fp.name[0]=='{' and fp.name[-1]=='}':
       try:                
         mapMat = eval(fp.name)
@@ -1567,6 +1569,56 @@ class Figure():
       for mat in (lstMat if fp.name=='*' else [ fp.geomGroup.getMaterial(fp.name), ]):
         self.addMaterial(PoserMaterial(src=mat), replace=True)
 
+
+  def __transformGeom(self, grp, fp:'FigurePart'):
+    ''' Create a copy of the grp with defined transformations: translation, symetry, rotations
+    
+    Parameters
+    ----------
+    fp: FigurePart
+      Description of the actor.
+      Used fields are: geomGroup, translation, rotation, orientation, center
+    
+    Return
+    ------
+    int,GeomGroup : Return Code, The GeomGroup (none if error)
+    
+    '''
+    ret = C_OK
+    
+    # Avoid Useless job
+    if not fp.rotation and not fp.translation:
+      return ret, grp
+    
+    # First do a deep copy of the group into a new optimized WaveGeom
+    nwg = grp.geom.copy(groups=grp)    
+    grp = nwg.getGroups()[0]
+    
+    AxisOrder, _ = fp.getOrientation(grp)
+      
+    try:
+      # Compute Actor's center
+      bc = fp.getBaryCentre(grp)
+  
+      # Manage Geomery Rotation first
+      if fp.rotation:
+        vri = fp.getRotation(grp)
+        grp.rotate(*vri, *bc, AxisOrder)
+              
+      # Manage Translation
+      if fp.translation:
+        tr = fp.getTranslation(self, bc, grp)
+        grp.translate(*tr)  
+        
+    except UsageError:
+      exc_type, exc_value, exc_tb = sys.exc_info()
+      traceback.print_exception(exc_type, exc_value, exc_tb)
+      # Input Error have already been managed and logged
+      ret = fp.ret
+      #ret = fp.status(C_FAIL, 'Computation error for [{:s}]:', grp.getName(), str(ve))
+
+    return ret, grp
+
     
  
   def createMechanical(self, lstParts:'list of FigurePart', optimize=True):
@@ -1650,7 +1702,7 @@ class Figure():
       if fp.partType[:3]=='act': # Actor Creation
         # Check level consistency
         if fp.logCond(fp.level<=0 or curLevelActor[fp.level-1] is None, \
-                      'Unconsistent level for part:{:s}', fp.name):
+                      f'Unconsistent level for part:{fp.name}'):
           continue
         
         act = PoserActor(Name=cleanName(fp.name) + idxSuffix)
@@ -1698,13 +1750,25 @@ class Figure():
             act.setGeomGroupName(fp.geomGroup.getName())
             # Copy the GeomGroup into the final geometry
             wg.addGroup(fp.geomGroup)
+            
+          elif fp.oplst and 'reuse' in fp.oplst: #  Reuse a Previously added/created geometry
+            # In such case, geomGroup is a string
+            # Check if the Geometry has really been created
+            realGrp = wg.getGroup(fp.geomGroup)
+            if not realGrp:
+              fp.status(C_FAIL, f'Actor [{fp.name}] has no group [{fp.geomGroup}]')
+            
+            act.setGeomType(PoserConst.GT_GLOBAL_OBJFILE)
+            act.setGeomGroupName(fp.geomGroup)
+            fp.geomGroup= realGrp
           else:
             # The Geometry will be loaded from another file
             act.setGeomType(PoserConst.GT_LOCAL_OBJFILE)
             act.setGeomFileName(fp.geom.getName())          
+            act.initBaseGeomCustom(fp.geom)
 
         # Create Default Channels
-        act.createDefaultChannels(fp) # fp.geomGroup, fp.trans, fp.rot, fp.orient, fp.center))
+        act.createDefaultChannels(fp)
         
       elif fp.partType[:3]=='geo': # Geometry add to previous actor's GeomGroup
         # Retrieve previous actor GeomGroup in the final geometry
@@ -1714,13 +1778,20 @@ class Figure():
         if fp.geom==srcWG and fullCopy:
           # The part should be in the global copied source
           partGeom = wg.getGroup(fp.geomGroup.getName())
-          prevGrp.fusion(partGeom)
+          # prevGrp.fusion(partGeom)
           if 'combine' in fp.oplst:
             # Delete the duplicate (the source geom)
             wg.removeGroup(partGeom)
             
         else: # The part comes from another WaveGeom
-          prevGrp.fusion(fp.geomGroup)
+          partGeom = fp.geomGroup
+          
+        # Do some transform(s) before fusion (do not alter the source)
+        ret, partGeom = self.__transformGeom(partGeom, fp)
+        
+        if ret==C_OK:
+          # Finaly fusion the result with the previous Group
+          prevGrp.fusion(partGeom)
 
       elif fp.partType[:3]=='mat': # Add a Material to the figure
         if isinstance(fp.geom, dict):
@@ -1735,13 +1806,13 @@ class Figure():
             self.__copyMat(fp, [ a for a in fp.geomGroup.getLstAttr() \
                           if a.getPoserType()==PoserToken.E_material ] )  
           else:
-            fp.status(C_FAIL, 'Actor [{:s}] has no custom material', fp.geomGroup.getName())
+            fp.status(C_FAIL, f'Actor [{fp.geomGroup.getName()}] has no custom material')
             
         else:
-          fp.status(C_FAIL, 'Actor [{:s}] illegal type', fp.geomGroup.getName())
+          fp.status(C_FAIL, f'Actor [{fp.geomGroup.getName()}] illegal type')
           
       else:
-        fp.status(C_FAIL, 'Unknow part type:{:s}', fp.partType)
+        fp.status(C_FAIL, f'Unknow part type:{fp.partType}')
     #End of for 'FigurePart'
     
     # Create file of the final geometry
