@@ -2,9 +2,9 @@
 # package: pftk.poser.kern
 from collections import namedtuple
 import logging
-import traceback, sys
+#import traceback, sys
 
-from langutil import RemoveExt, C_FAIL, C_OK, C_ERROR, C_FILE_NOT_FOUND
+from langutil import RemoveExt, C_FAIL, C_OK, C_ERROR, C_FILE_NOT_FOUND, UsageError
 from pypos3d.wftk.WFBasic import C_BAD_DELTAINDEX, C_BAD_DELTACOUNT, C_NODELTA, Vector3d, Point3d
 from pypos3d.wftk.PoserFileParser import PoserFileParser, ParsingErrorException
 from pypos3d.pftk.PoserBasic import PoserConst, PoserToken, PToken, buildRelPath, getRealPath, index, TAS, WBoolLine, create, Lang
@@ -311,7 +311,7 @@ class PoserMeshedObject(StructuredAttribut):
 
   
 
-  def createDefaultChannels(self, fp:'FigurePart'): # grp:'GeomGroup', translation=None, rotation=None, orientation=None, center=None):
+  def createDefaultChannels(self, fp:'FigurePart'):
     ''' Create the 16 minimal channels for this actor and initialize them according to the 4 textual parameters.
     Textual parameters are supposed to be 'stripped'
     
@@ -328,112 +328,40 @@ class PoserMeshedObject(StructuredAttribut):
     '''
     ret = C_OK
     
-    AxisOrder, tor = 'xyz', ''
-    orientation = fp.orientation
-    
-    if orientation:
-      pv = orientation.find(',')
-      AxisOrder = orientation[:pv].lower().strip() if pv>2 else orientation.lower().strip()
-
-      if AxisOrder in PoserToken.ORENTATIONS:
-        tor = orientation[pv+1:].strip() if pv>1 and pv<len(orientation) else ''
-      else:
-        AxisOrder = 'xyz'
-        tor = orientation.strip()
-      
-    # Get a copy of the default channels (we will modify the content)
-    lstChan = PoserToken.getMinimalChannels(AxisOrder, not fp.geomGroup)
     try:
+      AxisOrder, vOrient = fp.getOrientation()
+      
+      # Get a copy of the default channels (we will modify the content)
+      lstChan = PoserToken.getMinimalChannels(AxisOrder, not fp.geomGroup)
+      
       # Compute Actor's center
-      bc = Point3d()
-      center= fp.center
-      if center:
-        if center[0]=='(' and center[-1]==')': # Center is given by a Tuple of float
-          bc = Point3d(eval(center))
-        elif center.lower().startswith('face='): # Can be Face=Material name
-          centerFaceVx = fp.geomGroup.getFaceVertexBy(center[5:])
-          bc = Point3d.barycentre(centerFaceVx)
-        else:
-          return fp.status(C_FAIL,'Center definition error for [{:s}]:{:s}', self.getName(), center) 
-      else:
-        bc = fp.geomGroup.calcBarycentre()
+      bc = fp.getBaryCentre()
         
       # Set the Origin and Copy it in the static channels xOffset...
       self.setOrigin(bc)
       PoserToken.setInitVect(lstChan, PoserToken.OFFSETA, bc)
       PoserToken.setInitVect(lstChan, PoserToken.OFFSETB, Point3d(bc).neg())
   
-      # Compute Actor's orientation
-      rotx, roty, rotz = 0.0,0.0,0.0
-      if len(tor)>1:
-        
-        if tor[0]=='(' and tor[-1]==')': # The orientation is given by a Tuple of float
-          vr = Point3d(eval(tor))
-          rotx, roty, rotz = vr.x, vr.y, vr.z
-          
-        elif tor.lower().startswith('face='): # Can be Face=Material name
-          vxtab = fp.geomGroup.getFaceVertexBy(tor[5:])
-          rotx, roty, rotz = Point3d.triangle_orientation(vxtab[0], vxtab[1], vxtab[2], AxisOrder=AxisOrder, inDegrees=True)
-        else:
-          return fp.status(C_FAIL, 'Orientation definition error for [{:s}]:{:s}', self.getName(), tor)
-        
-      self.setOrientation(Vector3d(rotx, roty, rotz))
+      # Set Actor's orientation
+      self.setOrientation(vOrient)
       
       # Manage Translation
-      tr = Vector3d()
-      if fp.translation:
-        origin = Point3d()
-        
-        trl = fp.translation.lower()
-        pos = trl.find(':') # Potential location for a parent actor
-        if pos>0:
-          parentPMO = self.getFigure().findActor(fp.translation[pos+1:], withIndex=False)
-          if not parentPMO:
-            return fp.status(C_FAIL, 'Parent [{:s}] not found for [{:s}] translation', fp.translation[pos+1:], self.getName())
-
-          origin = parentPMO.getOrigin()
-        else:
-          parentPMO = None
-        
-        if trl[0]=='(' and trl[-1]==')': # Center is given by a Tuple of float
-          tr = Point3d(eval(trl))
-          
-        elif trl.startswith('face='): # Can be Face=Material name
-          centerFaceVx = fp.geomGroup.getFaceVertexBy(fp.translation[5:])
-          tr = Point3d.barycentre(centerFaceVx).sub(bc)
-        
-        # Manage symLL[:PartName]
-        elif trl.startswith('symxy'):
-          tr = Point3d( 0.0, 0.0, -2.0*(bc.z - origin.z))
-        elif trl.startswith('symyz'):
-          tr = Point3d(-2.0*(bc.x - origin.x), 0.0, 0.0)
-        elif trl.startswith('symzx'):
-          tr = Point3d( 0.0, -2.0*(bc.y - origin.y), 0.0)
-        else:
-          return fp.status(C_FAIL,'Translation definition error for [{:s}]:{:s}', self.getName(), fp.translation) 
-        
+      tr = fp.getTranslation(self.getFigure(), bc, actor=self)
       PoserToken.setInitVect(lstChan, PoserToken.TRANS, tr)
         
       # Manage Rotation
-      vri = Vector3d()
-      if fp.rotation:
-        if fp.rotation[0]=='(' and fp.rotation[-1]==')': # Center is given by a Tuple of float
-          vri = Vector3d(eval(fp.rotation))
-        else:
-          return fp.status(C_FAIL, 'Rotation definition error for [{:s}]:{:s}', self.getName(), fp.rotation)
-        
+      vri = fp.getRotation()
       PoserToken.setInitVect(lstChan, PoserToken.ROT, vri)
         
       # Create the list of channels
       for gtdef in lstChan:    
         vp = GenericTransform.createFrom(gtdef)
         self.addAttribut(vp)
-  
-    except ValueError as ve:
-      exc_type, exc_value, exc_tb = sys.exc_info()
-      traceback.print_exception(exc_type, exc_value, exc_tb)
-       
-      ret = fp.status(C_FAIL, 'Definition error for [{:s}]:', self.getName(), str(ve))
+
+    except UsageError:
+      #exc_type, exc_value, exc_tb = sys.exc_info()
+      #traceback.print_exception(exc_type, exc_value, exc_tb)       
+      ret = fp.ret
 
     return ret
 
@@ -475,6 +403,12 @@ class PoserMeshedObject(StructuredAttribut):
     else: # Alternate should not appear here
       s = PoserConst.GEOMPRINT[self._geomType]
     return s
+
+
+  def initBaseGeomCustom(self, wg:'WaveGeom'):
+    ''' Initialize the internal GeomCustom for a GT_LOCAL_OBJFILE '''
+    if self._geomType==PoserConst.GT_LOCAL_OBJFILE:
+      self._geomCustom = GeomCustom(src=wg)
   
   def getBaseGeomCustom(self, poserRootDir):
     ''' Return the Geometry definition of the object. Without applying any morph.
