@@ -573,10 +573,9 @@ class PoserMaterial(StructuredAttribut):
     self._ReflectionColor = WFColor(0.733333, 0.733333, 0.733333, 1.) 
     self._reflectionStrength = 1. 
     self._shaderTree = None
+    self.setPoserType(PoserToken.E_material) # FIX: Shall be set anyway
     
     if src:
-      self.setPoserType(PoserToken.E_material)
-          
       if isinstance(src, WFMat):
         # Create a Material from a simple WaveFront material
         self.setName(src.name)
