@@ -14,7 +14,7 @@ from pypos3d.wftk import WFBasic
 from pypos3d.wftk.WFBasic import C_VERTEX_NOT_FOUND, COUNT, FEPSILON, C_FACE_NORM, \
   C_FACE_TEXT, TexCoord2f, CoordSyst, C_FACE_MATMASK, C_FACE_TEXTNORM
 from pypos3d.wftk.WFBasic import LowestIdxPos, CreateLoop, FaceNormalOrder
-from pypos3d.wftk.WFBasic import Point3d, Vector3d, Edge, Regularity, IndexAdd, CreateTransform
+from pypos3d.wftk.WFBasic import Point3d, Vector3d, Edge, Regularity, IndexAdd, CreateTransform, MxProd
 from pypos3d.wftk.PaveList2D import PaveList2D
 
 
@@ -1368,8 +1368,255 @@ class GeomGroup(object):
     
     return self
   
-  
 
+  def translate(self, tx=0.0, ty=0.0, tz=0.0):
+    ''' Translate the group of the given vector.
+    This method creates new vertex (Point3d) in the coordList (usually in WaveGeom)
+    
+    Parameters
+    ----------
+    tx,ty,tz : Translation coordinates
+    
+    Return
+    ------
+    self for convenience
+    '''
+    # Create an empty mapping tables
+    nbInit = len(self.coordList)      
+    mapVert  = [-2] * nbInit
+
+    vertIdxSet = set( self.vertIdx )
+    
+    v = Point3d(tx,ty,tz)
+    nbPt = nbInit
+    # Create new translated points
+    for vi in vertIdxSet:
+      p = Point3d(self.coordList[vi])
+      p.add(v)
+      self.coordList.append(p)
+      mapVert[vi] = nbPt
+      nbPt += 1
+      
+    # Remap Group's vertex indexes with the new ones
+    self.vertIdx[:] = array.array('l', [ mapVert[vi] for vi in self.vertIdx ] )
+    
+    return self
+    
+  
+  def rotate(self, rx=0.0, ry=0.0, rz=0.0, cx=0.0, cy=0.0, cz=0.0, AxisOrder='xyz'):
+    ''' Rotate the group around standard axis with an optional rotation center and
+    according given rotation orders.    
+    Angles in degrees.
+    This method creates new vertex (Point3d) in the coordList (usually in WaveGeom)
+    
+    Parameters
+    ----------
+    rx,ry,rz : Axis rotation angles (float)
+    cx,cy,cz : Center coordinates
+    axisOrder : str, defaul 'xyz'
+      Rotation order between 'xyz', 'yzx' and 'zxy'
+    
+    Return
+    ------
+    self for convenience
+    
+    '''
+    # Create an empty mapping tables
+    nbInit = len(self.coordList)      
+    mapVert  = [-2] * nbInit
+
+    vertIdxSet = set( self.vertIdx )
+    
+    # Create the center for a simple origin usage
+    center = Point3d(cx,cy,cz)
+
+    theta = rx*math.pi/180.0
+    c, s = math.cos(theta), math.sin(theta)
+    MRotX = [ \
+       [ 1.0, 0.0, 0.0], \
+       [ 0.0,   c, - s ], \
+       [ 0.0,   s,   c ], \
+       ]
+  
+    theta = ry*math.pi/180.0
+    c, s = math.cos(theta), math.sin(theta)
+    MRotY = [ \
+       [   c, 0.0,   s ], \
+       [ 0.0, 1.0, 0.0 ], \
+       [ - s, 0.0,   c ], \
+       ]
+
+    theta = rz*math.pi/180.0
+    c, s = math.cos(theta), math.sin(theta)
+    MRotZ = [ \
+      [   c, - s, 0.0], \
+      [   s,   c, 0.0 ], \
+      [ 0.0, 0.0, 1.0 ], \
+      ]
+
+    if AxisOrder=='xyz':
+      m = MxProd(MRotY, MRotX)
+      m = MxProd(MRotZ, m)
+    elif AxisOrder=='yzx':
+      m = MxProd(MRotZ, MRotY)
+      m = MxProd(MRotX, m)
+    else: # zxy
+      m = MxProd(MRotX, MRotZ)
+      m = MxProd(MRotY, m)
+    
+    nbPt = nbInit
+    # Create new rotated points
+    for vi in vertIdxSet:
+      p = Point3d(self.coordList[vi])
+      p.sub(center)
+      p.inLin33(m)
+      p.add(center)
+      self.coordList.append(p)
+      mapVert[vi] = nbPt
+      nbPt += 1
+      
+    # Remap Group's vertex indexes with the new ones
+    self.vertIdx[:] = array.array('l', [ mapVert[vi] for vi in self.vertIdx ] )
+    
+    return self
+    
+  def symetry(self, cx=0.0, cy=0.0, cz=0.0, sym='symxy'):
+    ''' Mirror the group along standard planes with an optional origin center.
+
+    This method creates new vertex (Point3d) in the coordList (usually in WaveGeom)
+    
+    Parameters
+    ----------
+    cx,cy,cz : Center coordinates
+    sym : str, defaul 'symxy'
+      Plan for symetry 'symxy', 'symyz' and 'symzx'
+    
+    Return
+    ------
+    self for convenience
+    '''
+    # Create an empty mapping tables
+    nbInit = len(self.coordList)      
+    mapVert  = [-2] * nbInit
+
+    vertIdxSet = set( self.vertIdx )
+
+    nbPt = nbInit
+    # Create the center for a simple origin usage
+    c = Point3d(cx,cy,cz)
+    if sym=='symxy':
+      # Create new symetrized points
+      for vi in vertIdxSet:
+        p = Point3d(self.coordList[vi])
+        p.z = 2.0*c.z - p.z
+        self.coordList.append(p)
+        mapVert[vi] = nbPt
+        nbPt += 1
+      
+    elif sym=='symyz':
+      # Create new symetrized points
+      for vi in vertIdxSet:
+        p = Point3d(self.coordList[vi])
+        p.sub(c)
+        p.x = 2.0*c.x - p.x
+        p.add(c)
+        self.coordList.append(p)
+        mapVert[vi] = nbPt
+        nbPt += 1
+    else: # symzx
+      # Create new symetrized points
+      for vi in vertIdxSet:
+        p = Point3d(self.coordList[vi])
+        p.y = 2.0 * c.y - p.y 
+        self.coordList.append(p)
+        mapVert[vi] = nbPt
+        nbPt += 1
+    
+      
+    # Remap Group's vertex indexes with the new ones
+    self.vertIdx[:] = array.array('l', [ mapVert[vi] for vi in self.vertIdx ] )
+    
+    return self
+    
+    
+  def calcFaceOrientation(self, faceno=-1, materialName=None, AxisOrder='xyz'):
+    ''' Compute the orientation of a face according to an Axis order (xyz, yzx, zxy).
+    
+    The orientation is the set of rotations that aligns the first axis of the
+    axis order on the face normal vector.
+    
+    Parameters
+    ----------
+    faceno : int
+      Face to remove (if >0)
+
+    materialName : str
+      Name of the material. Use the first face with this material
+      
+    axisOrder : str, defaul 'xyz'
+      Rotation order between 'xyz', 'yzx' and 'zxy'
+
+    Returns
+    -------
+    int, Vector3d
+      C_OK : Success
+      C_FAIL : faceno==-1 and materialName not found
+      C_ERROR : Face Index Error
+    '''
+    if faceno==-1:
+      if materialName:
+        try:
+          matidx = self.lstMat.index(materialName)
+          faceno = self.findFace(matidx)
+          if faceno<0:
+            return C_FAIL, None
+        except ValueError:
+          return C_FAIL, None
+      else:
+        return C_ERROR, None
+    elif (faceno<0) or (faceno>=self.getNbFace()):
+      return C_ERROR, None
+
+    ret, vr = C_OK, None
+    #vxtab = self.geomGroup.getFaceVertexBy(tor[5:], raiseExcept=False)
+    vxtab = self.getFaceVertex(faceno)    
+    if vxtab and len(vxtab)>=3:
+      # fcenter, p1, p2 = vxtab[:3]
+      #
+      # # Try to be more accurate on non triangular faces
+      # if len(vxtab)>3:
+      #   fcenter = Point3d.barycentre(vxtab)
+      #   p1, p2 = vxtab[0], vxtab[1]
+      #   v0 = Vector3d.AB(fcenter, p1)
+      #   v1 = Vector3d.AB(fcenter, p2)
+      #   # Compute a first normale to get a sign reference
+      #   n0 = v0.cross(v1)
+      #
+      #   # Look for a point with a maximum (and better) absolute 'sinus'
+      #   bestnorm, maxs = n0, n0.norme2()
+      #   for p in vxtab[2:]:
+      #     v2 = Vector3d.AB(fcenter, p)
+      #     pv = v0.cross(v2)
+      #
+      #     n = pv.norme2()
+      #     if n>maxs:
+      #       p2 = p
+      #       bestnorm = pv
+      #       maxs = n
+      #
+      #   # If the best point generates a reserved normale, swap points order
+      #   if n0.dot(bestnorm) < 0.0:
+      #     p1, p2 = p2, p1
+      
+      # Perform the normal computation  
+      # vr = Vector3d( Point3d.triangle_orientation(fcenter, p1, p2, AxisOrder=axisOrder, inDegrees=True) )
+      vr = Point3d.face_orientation(vxtab, AxisOrder=AxisOrder, inDegrees=True)              
+    else:  
+      ret = C_FAIL # , msg = C_FAIL, f"Orientation error for [{refGrp.getName()}]: Missing Face {tor}"
+            
+    return ret, vr
+    
+      
   
   def unTriangularize(self, maxsin=FEPSILON):
     ''' Untriangularize the current group (inplace).
