@@ -259,8 +259,12 @@ class WaveGeom(object):
     self.texList = ntextab
 
 
-  def copy(self): # return WaveGeom wg
-    ''' Return a deep copy of self '''
+  def copy(self, groups=None):
+    ''' Return a deep copy of self, eventually restricted to given groups.
+    Parameters
+    ----------
+      groups : List of GeomGroup or List of Group names or a GeomGroup or a GeomGroup name to keep
+    '''
     wg = self.copyData()
 
     wg.setName(self._name)
@@ -270,8 +274,21 @@ class WaveGeom(object):
     wg.lstMat = [ s for s in self.lstMat ]
     wg._libMaterialName = self._libMaterialName
 
+    lstGrps = []
+
+    if groups:
+      if isinstance(groups, str):
+        lstGrps.append(self.getGroup(groups))
+      elif isinstance(groups, GeomGroup):
+        lstGrps.append(groups)
+      else:
+        lstGrps = [ (g if isinstance(g, GeomGroup) else self.getGroup(g)) for g in lstGrps ]
+      
+    else:
+      lstGrps = self.groups
+      
     # Each face's Geometry Group membership is kept here. . .
-    wg.groups = [ GeomGroup(src=ggd, inCoordList=wg.coordList) for ggd in self.groups ]
+    wg.groups = [ GeomGroup(src=ggd, inCoordList=wg.coordList) for ggd in lstGrps ]
 
     # Link correctly groups
     # FIX a major error
@@ -1087,13 +1104,10 @@ class WaveGeom(object):
 
   # Calculate for a Group the mapping table for Vertex 
   # beween group index and global GeomCustom index.
-  #
   #@return    the mapping table. 
   #           res[i] is the index in the GeomCustom vertex table.
   #  public int[] calcGroupVertIndex(GeomGroup g)
   def calcGroupVertIndex(self, grp):
-    #     for faceno in range(0, grp.getNbFace()):
-    #       svi |= set(grp.vertIdx[grp.stripCount[faceno]:grp.stripCount[faceno + 1]])
     svi = set( grp.vertIdx )
     nVertIdx = list(svi)
     nVertIdx.sort()
